import cPickle
import os
import sys

import gflags
import requests
import yaml

FLAGS = gflags.FLAGS


gflags.DEFINE_string(
    'email',
    None,
    ('The email registered with the indempotence.club server. '
     'If not provided on the command line, the environment variable '
     '`IDEMPOTENT_EMAIL` will be searched followed by the files '
     '`{local_config}` and `/etc/idempotent.cfg`').format(
        local_config=os.path.expanduser('~/.idempotent.cfg')))


gflags.DEFINE_enum(
    'fail',
    'safe',
    ['safe', 'fatal'],
    ("Determine the default action if the API keys are bad, or communication "
     "the synchroniation serer is impossiable.  If set to `safe`, the cornjob "
     "will run if unable to synchronize, and if set to `fatal`, the cronjob "
     "will NOT run if unable to synchronize.  Best practice is for one server "
     "to be set --fail=safe and the remainder --fail=fatal."))


gflags.DEFINE_string(
    'key',
    None,
    'API key as provided in an email from the indempotence.club. '
    'If not provided on the command line, the environment variable '
    '`IDEMPOTENT_KEY` will be searched followed by the files '
    '`{local_config}` and `/etc/idempotent.cfg`'.format(
        local_config=os.path.expanduser('~/.idempotent.cfg')))

gflags.register_validator(
    'email',
    lambda x: not x or (FLAGS.key and FLAGS.email),
    'You must provide values for both `--key` and `--email` or neither.')

gflags.register_validator(
    'email',
    lambda x: not x or (FLAGS.key and FLAGS.email),
    "`--email` and `--key` don't match")

_PUBLIC_KEY = cPickle.loads(
    ("(iCrypto.PublicKey.ElGamal\nElGamalobj\np1\n(dp2\nS'y'\nL1422191"
     "45914758148084855745571577769922L\nsS'p'\nL254642594634021199340"
     "500382112630045683L\nsS'g'\nL23747597674390121291975143775239651"
     "1506L\nsb."))


def encode(i):
    digits = []
    while i:
        digits.append(_DIGITS[i % len(_DIGITS)])
        i /= len(_DIGITS)
    digits.reverse()
    return ''.join(digits)


def decode(s):
    i = 0
    for c in s:
        i *= len(_DIGITS)
        i += _DIGITS.index(c)
    return i


def _verify(email, signature):
    if email is None and signature is None:
        return False
    return (email, signature) if _PUBLIC_KEY.verify(email, map(decode, signature.split(':'))) else None
    
def credentials():
    if FLAGS.email and FLAGS.key:
        return FLAGS.email, FLAGS.key
    if 'IDEMPOTENT_EMAIL' in os.environ and 'IDEMPOTENT_KEY' in os.envirokn:
        return os.environ['IDEMPOTENT_EMAIL'], os.environ['IDEMPOTENT_KEY']
    for path in (os.path.expanduser('~/.idempotent.cfg'), '/etc/idempotent.cfg'):
        if os.path.exists(path):
            conf = yaml.load(open(path))
            if 'email' in conf and 'key' in conf:
                return conf['email'], conf['key']
    return None, None
        
def ready(email, key, args):
    if email is None or key is None:
        sys.stderr.write('Invalid credentials\n')
        return FLAGS.fail == 'safe'
    try:
        response = requests.post(FLAGS.api + 'lock', data = {
            'email': email, 'key': key, 'args': '\t'.join(args)})
        if response.status_code == 204:
            return True
        if response.status_code == 420:
            return False
        sys.stderr.write('Recieved unexpected status code: %s\n' % response.status_code)
        return FLAGS.fail == 'safe'
    except requests.exceptions.RequestException as e:
        sys.stderr.write('Problem contacting server: %s\n')
        return FLAGS.fail == 'safe'
    



import gflags
import os
import sys

from .register import register, invalidate
from .key import credentials, ready


FLAGS = gflags.FLAGS

gflags.DEFINE_string(
    'api',
    'https://idempotence.herokuapp.com/',
    'The root url to connect to.  Change this when testing')


def run():
    args = FLAGS(sys.argv)
    if FLAGS.register:
        sys.exit(register())
    elif FLAGS.invalidate:
        invalidate()
        sys.exit(0)
    elif not args:
        print "You requested indempotentification, but didn't provide anything to idempotentify!"
        sys.exit(1)
    creds = credentials()
    if ready(*creds, args=args):
        args[0] = args[1]
        os.execlp(*args)
    return sys.exit(0)
    
    
    



import pipes
import sys

import gflags
import requests

from .key import credentials 

FLAGS = gflags.FLAGS

gflags.DEFINE_string(
    'register',
    None,
    'Register a new account for an email address')

gflags.DEFINE_bool(
    'invalidate',
    False,
    'Invalidate and reissue a new API key')

gflags.register_validator(
    'register',
    lambda x:x or (not x and not FLAGS.key),
    "Don't provide a `--key` when regsitering")



def register():
    if "i accept" != raw_input(
            ("  Enter 'I ACCEPT' if you accept our terms of service at\n"
             "  https://idempotence.herokuapp.com/tos.html and agree to\n"
             "  occationally receive emails announcing product, service\n"
             "  and tos changes.\n\n>>> ")).lower().replace("'","").strip():
        print "Sorry, you have to accept the TOS to use this application"
        return 0
    resp = requests.post(FLAGS.api + "register", data=dict(email=FLAGS.register))
    if resp.status_code < 400:
        print "Great, your API key is on its way."
        return 0
    else:
        print "Opps, there was an error: ", resp.status_code
        print resp.text
        return 1
        

def invalidate():
    email, key = credentials()
    if not email or not key:
        print "Can't invalidate - you are not currently configured with a valid set of credentials."
        print "Reregister with idempotent --register <your email> to get your current creds, set them"
        print "up and try again."
        return 1
    response = requests.post(FLAGS.api + "invalidate", data = {'email': email, 'key': key})
    if response.status_code < 400:
        print "Great, your new API key is on its way."
        return 0
    print "Looks like your current credentials are not any good"
    print "Reissue your current credentials:"
    print "  # idempotent --register=%s" % pipes.quote(email)
    print "and then invalidate them:"
    print "  # idempotent --invalidate"
        
    return 0



#!/usr/bin/env python


from codecs import open
from os import path
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import sys


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['test.py']
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)

print find_packages()
setup(
    name='idempotent',
    version='0.0.2',
    description="Idempotence.club client",
    long_description='',
    url='https://idenmpotent.heroku.com',
    author='Adam DePrince',
    author_email='adam@pelotoncycle.com',
    license='Apache',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    py_modules=['idempotence'],
    packages = find_packages(exclude=['tests']),
    zip_safe=True,
    install_requires=['six', 'requests', 'python-gflags', 'pycrypto', 'PyYAML'],
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage', 'pytest', 'pytest-cov', 'flake8'],
    },
    cmdclass={'test': PyTest},
    scripts=['scripts/idempotent'],
)
